﻿using Syncfusion.UI.Xaml.Charts;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections;

namespace Sample1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void NumericalAxis_ActualRangeChanged(object sender, ActualRangeChangedEventArgs e)
        {
            //var axis = sender as ChartAxisBase2D;
            //var minimum = (double)e.ActualMinimum;
            //var maximum = (double)e.ActualMaximum;
            //var delta = maximum - minimum;

            //// Calculate the axis position according to the zoomposition and zoomfactor
            //double start = minimum + axis.ZoomPosition * delta;
            //double end = start + axis.ZoomFactor * delta;

            //e.VisibleMinimum = start;
            //e.VisibleMaximum = end;
        }

        // Since the range is manually changed while resetting the range has to be set to it's default position.
        private void Chart_ResetZooming(object sender, ResetZoomEventArgs e)
        {
            var axis = e.Axis as NumericalAxis;

            var axisOrientation = (Orientation)axis.GetType().GetProperty("Orientation", System.Reflection.BindingFlags.GetProperty | System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic).GetValue(axis);
            var viewModel = (sender as SfChart).DataContext as ViewModel;

            if (axisOrientation == Orientation.Horizontal)
            {
                axis.Minimum = viewModel.XAxisRange.Start;
                axis.Maximum = viewModel.XAxisRange.End;
            }
            else
            {
                axis.Minimum = viewModel.YAxisRange.Start;
                axis.Maximum = viewModel.YAxisRange.End;
            }
        }
    }

    public class Model
    {
        public double XValue { get; set; }
        public double YValue { get; set; }
    }

    // Since we manipulating the ranges manually. Initially set the default x range and y range or calculate from the date points provided.
    public class ViewModel
    {
        public ViewModel()
        {
            GenerateData();

            XAxisRange = new DoubleRange(0, 5);
            YAxisRange = new DoubleRange(0, 50);
        }

        public DoubleRange XAxisRange { get; set; }

        public DoubleRange YAxisRange { get; set; }

        public void GenerateData()
        {
            Data = new ObservableCollection<Model>();
            Random rd = new Random();
            for (int i = 0; i < 6; i++)
            {
                Data.Add(new Model()
                {
                    XValue = i,
                    YValue = rd.Next(0, 50)
                });
            }
        }

        private ObservableCollection<Model> data;

        public ObservableCollection<Model> Data
        {
            get { return data; }
            set { data = value;  }
        }      
    }

    /// <summary>
    /// Custom zoom pan behavior class is created to calculate the zooming as required for our zoom in and zoom out operations.
    /// </summary>
    public class CustomChartZoomPanBehavior : ChartZoomPanBehavior
    {
        private int mouseWheelDirection;
        
        protected override void OnMouseWheel(MouseWheelEventArgs e)
        {
            mouseWheelDirection = e.Delta > 0 ? 1 : -1;
            base.OnMouseWheel(e);
        }
                
        // Overridng the zoom method to calculate the zooming position in zoom out and zoom in operations.
        public override bool Zoom(double cumulativeScale, double origin, ChartAxisBase2D axis)
        {
            double currentScale = Math.Max(1 / ChartMath.MinMax(axis.ZoomFactor, 0, 1), 1);
            cumulativeScale = currentScale + (0.25 * mouseWheelDirection);

            if (axis != null)
            {
                double calcZoomPos = 0;
                double calcZoomFactor = 0;
                double previousPosition = axis.ZoomPosition;
                double previousFactor = axis.ZoomFactor;

                // Calculating zoomposition and zoomfactor.
                calcZoomFactor = 1 / cumulativeScale;
                calcZoomPos = previousPosition + ((previousFactor - calcZoomFactor) * origin);

                var newZoomFactor = (calcZoomPos + calcZoomFactor) > 1 ? 1 - calcZoomPos : calcZoomFactor;
                var axisActualRange = (DoubleRange)axis.GetType().GetProperty("ActualRange", System.Reflection.BindingFlags.GetProperty | System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic).GetValue(axis);

                // Calculate the axis position according to the zoomposition and zoomfactor
                double start = axisActualRange.Start + calcZoomPos * axisActualRange.Delta;
                double end = start + calcZoomFactor * axisActualRange.Delta;

                (axis as NumericalAxis).Minimum = start;
                (axis as NumericalAxis).Maximum = end;
            }

            return false;
        }
    }       
}
